@@ -17,12 +17,12 @@ logging.getLogger("numexpr").setLevel(logging.WARNING)
 # Custom functions
 from .utils import ref_species_options, find_latest_ens_rel
 
-from .constants import ENSEMBL_FTP_URL
+from .constants import ENSEMBL_FTP_URL, ENSEMBL_FTP_URL_PLANT
 
 
 def ref(species, which="all", release=None, ftp=False, save=False, list_species=False):
     """
-    Fetch FTPs for reference genomes and annotations by species.
+    Fetch FTPs for reference genomes and annotations by species from Ensembl.
 
     Args:
     - species       Defines the species for which the reference should be fetched in the format "<genus>_<species>",
@@ -37,12 +37,12 @@ def ref(species, which="all", release=None, ftp=False, save=False, list_species=
                     'cdrna' - Returns transcript sequences corresponding to non-coding RNA genes (ncRNA).
                     'pep' - Returns the protein translations of Ensembl genes.
     - release       Defines the Ensembl release number from which the files are fetched, e.g. release = 104.
-                    (Ensembl releases earlier than release 48 are not supported.)
-                    By default, the latest Ensembl release is used.
+                    Default: None -> latest Ensembl release is used
     - ftp           Return only the requested FTP links in a list (default: False).
     - save          Save the results in the local directory (default: False).
 
-    - list_species  If True and `species=None`, returns a list of all available species (default: False).
+    - list_species  If True and `species=None`, returns a list of all available species from the Ensembl database for large genomes
+                    (not including plants/bacteria) (default: False).
                     (Can be combined with `release` to get the available species from a specific Ensembl release.)
 
     Returns a dictionary containing the requested URLs with their respective Ensembl version and release date and time.
@@ -81,26 +81,17 @@ def ref(species, which="all", release=None, ftp=False, save=False, list_species=
     # In case species was passed with upper case letters
     species = species.lower()
 
+    ## For plants and bacteria, switch to other databases
+    # Check if species in plant or bacteria Ensembl database
+    if species in ref_species_options(
+        "dna", database=ENSEMBL_FTP_URL_PLANT, release=None
+    ):
+        database = ENSEMBL_FTP_URL_PLANT
+    else:
+        database = ENSEMBL_FTP_URL
+
     ## Find latest Ensembl release
-    url = ENSEMBL_FTP_URL
-    html = requests.get(url)
-
-    # Raise error if status code not "OK" Response
-    if html.status_code != 200:
-        raise RuntimeError(
-            f"Ensembl FTP site returned error status code {html.status_code}. Please try again."
-        )
-
-    soup = BeautifulSoup(html.text, "html.parser")
-    # Find all releases
-    releases = soup.body.findAll(text=re.compile("release-"))
-    # Get release numbers
-    rels = []
-    for rel in releases:
-        rels.append(rel.split("/")[0].split("-")[-1])
-
-    # Find highest release number (= latest release)
-    ENS_rel = np.array(rels).astype(int).max()
+    ENS_rel = find_latest_ens_rel(database)
 
     # If release != None, use user-defined Ensembl release
     if release != None:
@@ -114,9 +105,9 @@ def ref(species, which="all", release=None, ftp=False, save=False, list_species=
 
     ## Raise error if species not found
     # Find all available species for GTFs for this Ensembl release
-    species_list_gtf = ref_species_options("gtf", release=ENS_rel)
+    species_list_gtf = ref_species_options("gtf", database=database, release=ENS_rel)
     # Find all available species for FASTAs for this Ensembl release
-    species_list_dna = ref_species_options("dna", release=ENS_rel)
+    species_list_dna = ref_species_options("dna", database=database, release=ENS_rel)
 
     # Find intersection of the two lists
     # (Only species which have GTF and FASTAs available can continue)
@@ -126,12 +117,12 @@ def ref(species, which="all", release=None, ftp=False, save=False, list_species=
         raise ValueError(
             f"Species does not match any available species for Ensembl release {ENS_rel}.\n"
             "Please double-check spelling.\n"
-            "'gget ref --list' -> lists out all available species (Python: 'gget.ref(None, list_species=True)').\n"
+            "'gget ref --list_species' -> lists out all available species (Python: 'gget.ref(None, list_species=True)').\n"
             "Combine with `release` argument to define specific Ensembl release (default: latest).\n"
         )
 
     ## Get GTF link for this species and release
-    url = ENSEMBL_FTP_URL + f"release-{ENS_rel}/gtf/{species}/"
+    url = database + f"release-{ENS_rel}/gtf/{species}/"
     html = requests.get(url)
 
     # Raise error if status code not "OK" Response
@@ -160,13 +151,13 @@ def ref(species, which="all", release=None, ftp=False, save=False, list_species=
             # Get release date and time from <None> elements (since there are twice as many, 2x and +1 to move from string to date)
             gtf_date_size = nones[i * 2 + 1]
 
-    gtf_url = ENSEMBL_FTP_URL + f"release-{ENS_rel}/gtf/{species}/{gtf_str['href']}"
+    gtf_url = database + f"release-{ENS_rel}/gtf/{species}/{gtf_str['href']}"
 
     gtf_date = gtf_date_size.strip().split("  ")[0]
     gtf_size = gtf_date_size.strip().split("  ")[-1]
 
     ## Get cDNA FASTA link for this species and release
-    url = ENSEMBL_FTP_URL + f"release-{ENS_rel}/fasta/{species}/cdna"
+    url = database + f"release-{ENS_rel}/fasta/{species}/cdna"
     html = requests.get(url)
 
     # Raise error if status code not "OK" Response
@@ -195,15 +186,13 @@ def ref(species, which="all", release=None, ftp=False, save=False, list_species=
             # Get release date and time from <None> elements (since there are twice as many, 2x and +1 to move from string to date)
             cdna_date_size = nones[i * 2 + 1]
 
-    cdna_url = (
-        ENSEMBL_FTP_URL + f"release-{ENS_rel}/fasta/{species}/cdna/{cdna_str['href']}"
-    )
+    cdna_url = database + f"release-{ENS_rel}/fasta/{species}/cdna/{cdna_str['href']}"
 
     cdna_date = cdna_date_size.strip().split("  ")[0]
     cdna_size = cdna_date_size.strip().split("  ")[-1]
 
     ## Get DNA FASTA link for this species and release
-    url = ENSEMBL_FTP_URL + f"release-{ENS_rel}/fasta/{species}/dna"
+    url = database + f"release-{ENS_rel}/fasta/{species}/dna"
     html = requests.get(url)
 
     # Raise error if status code not "OK" Response
@@ -242,9 +231,7 @@ def ref(species, which="all", release=None, ftp=False, save=False, list_species=
                 # Get date from non-assigned values (since there are twice as many, 2x and +1 to move from string to date)
                 dna_date_size = nones[i * 2 + 1]
 
-    dna_url = (
-        ENSEMBL_FTP_URL + f"release-{ENS_rel}/fasta/{species}/dna/{dna_str['href']}"
-    )
+    dna_url = database + f"release-{ENS_rel}/fasta/{species}/dna/{dna_str['href']}"
 
     dna_date = dna_date_size.strip().split("  ")[0]
     dna_size = dna_date_size.strip().split("  ")[-1]
@@ -253,7 +240,7 @@ def ref(species, which="all", release=None, ftp=False, save=False, list_species=
     dna_size = dna_size.strip()
 
     ## Get CDS FASTA link for this species and release
-    url = ENSEMBL_FTP_URL + f"release-{ENS_rel}/fasta/{species}/cds"
+    url = database + f"release-{ENS_rel}/fasta/{species}/cds"
     html = requests.get(url)
 
     # Raise error if status code not "OK" Response
@@ -282,52 +269,52 @@ def ref(species, which="all", release=None, ftp=False, save=False, list_species=
             # Get release date and time from <None> elements (since there are twice as many, 2x and +1 to move from string to date)
             cds_date_size = nones[i * 2 + 1]
 
-    cds_url = (
-        ENSEMBL_FTP_URL + f"release-{ENS_rel}/fasta/{species}/cds/{cds_str['href']}"
-    )
+    cds_url = database + f"release-{ENS_rel}/fasta/{species}/cds/{cds_str['href']}"
 
     cds_date = cds_date_size.strip().split("  ")[0]
     cds_size = cds_date_size.strip().split("  ")[-1]
 
-    ## Get ncRNA FASTA link for this species and release
-    url = ENSEMBL_FTP_URL + f"release-{ENS_rel}/fasta/{species}/ncrna"
+    ## Get ncRNA FASTA link for this species and release (if available)
+    url = database + f"release-{ENS_rel}/fasta/{species}/ncrna"
     html = requests.get(url)
 
-    # Raise error if status code not "OK" Response
-    if html.status_code != 200:
-        raise RuntimeError(
-            f"HTTP response status code {html.status_code}. Please try again.\n"
+    # If ncRNA data is not available, HTML requests returns an error code (!= 200)
+    if html.status_code == 200:
+        soup = BeautifulSoup(html.text, "html.parser")
+
+        # The url can be found under an <a> object tag in the html,
+        # but the date and size do not have an object tag (element=None)
+        nones = []
+        a_elements = []
+        pre = soup.find("pre")
+        for element in pre.descendants:
+            if element.name == "a":
+                a_elements.append(element)
+            elif element.name != "a":
+                nones.append(element)
+
+        # Find the <a> element containing the url
+        for i, string in enumerate(a_elements):
+            if ".ncrna.fa" in string["href"]:
+                ncrna_str = string
+                # Get release date and time from <None> elements (since there are twice as many, 2x and +1 to move from string to date)
+                ncrna_date_size = nones[i * 2 + 1]
+
+        ncrna_url = (
+            database + f"release-{ENS_rel}/fasta/{species}/ncrna/{ncrna_str['href']}"
         )
 
-    soup = BeautifulSoup(html.text, "html.parser")
+        ncrna_date = ncrna_date_size.strip().split("  ")[0]
+        ncrna_size = ncrna_date_size.strip().split("  ")[-1]
 
-    # The url can be found under an <a> object tag in the html,
-    # but the date and size do not have an object tag (element=None)
-    nones = []
-    a_elements = []
-    pre = soup.find("pre")
-    for element in pre.descendants:
-        if element.name == "a":
-            a_elements.append(element)
-        elif element.name != "a":
-            nones.append(element)
-
-    # Find the <a> element containing the url
-    for i, string in enumerate(a_elements):
-        if ".ncrna.fa" in string["href"]:
-            ncrna_str = string
-            # Get release date and time from <None> elements (since there are twice as many, 2x and +1 to move from string to date)
-            ncrna_date_size = nones[i * 2 + 1]
-
-    ncrna_url = (
-        ENSEMBL_FTP_URL + f"release-{ENS_rel}/fasta/{species}/ncrna/{ncrna_str['href']}"
-    )
-
-    ncrna_date = ncrna_date_size.strip().split("  ")[0]
-    ncrna_size = ncrna_date_size.strip().split("  ")[-1]
+    # If the HTML request returned an error code here, I will assume that ncRNA data is not available
+    else:
+        ncrna_url = ""
+        ncrna_date = " "
+        ncrna_size = ""
 
     ## Get pep FASTA link for this species and release
-    url = ENSEMBL_FTP_URL + f"release-{ENS_rel}/fasta/{species}/pep"
+    url = database + f"release-{ENS_rel}/fasta/{species}/pep"
     html = requests.get(url)
 
     # Raise error if status code not "OK" Response
@@ -356,9 +343,7 @@ def ref(species, which="all", release=None, ftp=False, save=False, list_species=
             # Get release date and time from <None> elements (since there are twice as many, 2x and +1 to move from string to date)
             pep_date_size = nones[i * 2 + 1]
 
-    pep_url = (
-        ENSEMBL_FTP_URL + f"release-{ENS_rel}/fasta/{species}/pep/{pep_str['href']}"
-    )
+    pep_url = database + f"release-{ENS_rel}/fasta/{species}/pep/{pep_str['href']}"
 
     pep_date = pep_date_size.strip().split("  ")[0]
     pep_size = pep_date_size.strip().split("  ")[-1]
